#include <File.au3>

#include '..\..\..\..\_build\include\MiscLib.au3'

Global Enum $FUNCTION_PARAMS_TITLE, _
		$FUNCTION_PARAMS_DESCRIPTION, _
		$FUNCTION_PARAMS_INCLUDEORSYNTAX, _
		$FUNCTION_PARAMS_SYNTAX, _
		$FUNCTION_PARAMS_MAX
Global Const $STR_STRINGSPLIT = BitOR($STR_ENTIRESPLIT, $STR_NOCOUNT)

GenerateSyntaxLists()

; ------------------------------------------------------------------------------
; Wrapper function for creating the syntax lists.
; ------------------------------------------------------------------------------
Func GenerateSyntaxLists()
	Local Const $FUNCTIONLIST = @ScriptDir & '\functions.txt', $FUNCTIONLIST_FULL = @ScriptDir & '\function_full.txt', _
			$KEYWORDLIST = @ScriptDir & '\keywords.txt', _
			$LIBFUNCTIONLIST = @ScriptDir & '\libfunctions.txt', $LIBFUNCTIONLIST_FULL = @ScriptDir & '\libfunctions_full.txt', _
			$MACROLIST = @ScriptDir & '\macros.txt', $MACROLIST_FULL = @ScriptDir & '\macros_full.txt'

	; Generate the native fucntions, UDF functions and keywords lists.
	_GenerateFunctions($HELP_TYPE_NATIVE, $FUNCTIONLIST, @ScriptDir & '\function_params.txt', $FUNCTIONLIST_FULL)
	_GenerateFunctions($HELP_TYPE_USERDEFINED, $LIBFUNCTIONLIST, @ScriptDir & '\libfunction_params.txt', $LIBFUNCTIONLIST_FULL)

	If Not FileExists($KEYWORDLIST) Then
		_GenerateKeywords($KEYWORDLIST)
	EndIf

	; Generate the macros lists.
	_GenerateMacros(@ScriptDir & '\..\..\html\macros', $MACROLIST, $MACROLIST_FULL)

	Return True
EndFunc   ;==>GenerateSyntaxLists

; ------------------------------------------------------------------------------
; Generate a list of the functions and their parameters. Designed for native and UDF functions.
; ------------------------------------------------------------------------------
Func _GenerateFunctions($iType, $sFunctionListOut, $sFunctionParamsOut, $sFunctionFullOut)
	Local $sHelpFileTxtPath = '', $sType = ''
	; Set the filepaths based on the type passed.
	Switch $iType
		Case $HELP_TYPE_NATIVE
			$sHelpFileTxtPath = @ScriptDir & '\..\txtFunctions'
			$sType = $HELP_SECTION_FUNCTION

		Case $HELP_TYPE_USERDEFINED
			$sHelpFileTxtPath = @ScriptDir & '\..\txtlibfunctions'
			$sType = $HELP_SECTION_USERDEFINED

	EndSwitch
	$sHelpFileTxtPath = _GetFullPath($sHelpFileTxtPath)

	Local $aFileList = _FileListToArray($sHelpFileTxtPath, '*.txt', $FLTA_FILES, True)
	If @error Then Return False

	Local $aArray = 0
	Local Static $sAlternativeTitles = 0
	If Not IsString($sAlternativeTitles) Then
		$sAlternativeTitles = @CRLF
		$aArray = IniReadSection(@ScriptDir & '\AutoIt Extractor.ini', 'Aliases')
		If Not @error Then
			Local Enum $eIni_Key, $eIni_Value
			#forceref $eIni_Key
			For $i = 1 To UBound($aArray) - 1
				$sAlternativeTitles &= $aArray[$i][$eIni_Value] & @CRLF
			Next
		EndIf
	EndIf

	Local Const $DOLLAR_LENGTH = StringLen('$')
	Local $aTitle = 0, _
			$sAdditionalTitles = '', $sData = '', $sDescription = '', $sFileName = '', $sFunctionList = '', $sFunctionListParams = '', $sFunctionListFull = '', $sSyntaxParams = ''
	For $i = 1 To UBound($aFileList) - 1 ; Create a list of functions and function parameters.
		$sFileName = $aFileList[$i]
		$sFileName = StringTrimLeft($sFileName, StringInStr($sFileName, '\', $STR_NOCASESENSEBASIC, -1)) ; Trim the filepath by using the position of \.
		$sFileName = StringLeft($sFileName, StringInStr($sFileName, '.', $STR_NOCASESENSEBASIC, -1) - 1) ; Trim the extension.
		If StringLeft($sFileName, $DOLLAR_LENGTH) = '$' Then ContinueLoop ; Skip $tag constants.

		$sData = FileRead($aFileList[$i])
		_StripEmptyLines($sData)
		_StripWhitespace($sData)

		$sDescription = ''
		$sSyntaxParams = ''

		; Parse the TXT doc's description and syntax lines.
		$aArray = StringRegExp($sData, '^(?:' & $sType & '\R(\V*)\R*' & $HELP_SECTION_DESCRIPTION & '\R(\V*)\R*' & $HELP_SECTION_SYNTAX & '\R(\V*)\R(\V*))', $STR_REGEXPARRAYGLOBALMATCH)
		If Not @error And UBound($aArray) = $FUNCTION_PARAMS_MAX Then
			$sDescription &= $aArray[$FUNCTION_PARAMS_DESCRIPTION]
			_ConvertHTMLEntityToSymbols($sDescription)
			If $iType = $HELP_TYPE_USERDEFINED Then
				$sDescription &= ' (Requires: ' & $aArray[$FUNCTION_PARAMS_INCLUDEORSYNTAX] & ')'
				$sSyntaxParams = $aArray[$FUNCTION_PARAMS_SYNTAX]
			Else
				$sSyntaxParams = $aArray[$FUNCTION_PARAMS_INCLUDEORSYNTAX]
			EndIf

			$aTitle = StringRegExp($sAlternativeTitles, '(?im:^\Q' & $aArray[$FUNCTION_PARAMS_TITLE] & '\E\h*\->\h*(\V+)$)', $STR_REGEXPARRAYGLOBALMATCH)
			If Not @error Then ; Check if an alternative exists and append to the data strings.
				$sAdditionalTitles &= $aTitle[0] & @CRLF
				$aArray[$FUNCTION_PARAMS_TITLE] = StringRegExpReplace($aArray[$FUNCTION_PARAMS_TITLE], '^(\w+)\V*$', '\1') ; Strip everything but the initial function.
			EndIf

			$sFunctionList &= $aArray[$FUNCTION_PARAMS_TITLE] & @CRLF ; Function list.
			$sFunctionListParams &= $sSyntaxParams & ' ' & $sDescription & @CRLF ; Function parameter list.
			$sFunctionListFull &= $aArray[$FUNCTION_PARAMS_TITLE] & $ASCII_ACK & $sSyntaxParams & $ASCII_ACK & $sDescription & $ASCII_ACK & @CRLF ; Function full list.
		EndIf
	Next

	$sFunctionList &= $sAdditionalTitles

	_StripEmptyLinesEndOfFile($sFunctionList) ; Strip EOLs.
	_StripEmptyLinesEndOfFile($sFunctionListParams) ; Strip EOLs.
	_StripEmptyLinesEndOfFileACK($sFunctionListFull) ; Strip EOLs plus ACK.

	; Write the output.
	_FileWrite($sFunctionListOut, $sFunctionList)

	; Write the output.
	_FileWrite($sFunctionParamsOut, $sFunctionListParams)

	; Write the output for the full list of functions<ACK>params<ACK>description<ACK>CRLF. Chr(6) or ACK is used as a delimiter.
	_FileWrite($sFunctionFullOut, $sFunctionListFull)

	Return True
EndFunc   ;==>_GenerateFunctions

; ------------------------------------------------------------------------------
; Generate a list of keywords.
; ------------------------------------------------------------------------------
Func _GenerateKeywords($sKeywordListOut)
	Local $hKeywords = 0
	If _AssociativeArray_Startup($hKeywords, True) Then
		Local $sHelpFileTxtPath = _GetFullPath(@ScriptDir & '\..\txtKeywords'), $sType = $HELP_SECTION_KEYWORD

		Local $aFileList = _FileListToArray($sHelpFileTxtPath, '*.txt', $FLTA_FILES, True)
		If @error Then Return False

		; These are a list of exceptions that aren't found in the txtKeywords directory.
		Local $aKeywords = StringSplit(IniRead(@ScriptDir & '\AutoIt Extractor.ini', 'Keywords', 'List', ''), ' ', $STR_STRINGSPLIT), _
				$aPreProcessor = StringSplit(IniRead(@ScriptDir & '\AutoIt Extractor.ini', 'PreProcessor', 'List', ''), ' ', $STR_STRINGSPLIT), _
				$aSpecial = StringSplit(IniRead(@ScriptDir & '\AutoIt Extractor.ini', 'Special', 'List', ''), ' ', $STR_STRINGSPLIT)

		; Add the exceptions.
		For $i = 0 To UBound($aKeywords) - 1
			$hKeywords.Item($aKeywords[$i]) = $aKeywords[$i] ; Keyword list.
		Next
		For $i = 0 To UBound($aPreProcessor) - 1
			$hKeywords.Item($aPreProcessor[$i]) = $aPreProcessor[$i] ; PreProcessor list.
		Next
		For $i = 0 To UBound($aSpecial) - 1
			$hKeywords.Item($aSpecial[$i]) = $aSpecial[$i] ; Special list.
		Next

		Local $aArray = 0, _
				$sData = ''
		For $i = 1 To UBound($aFileList) - 1 ; Create a list of Keywords and Keyword parameters.
			$sData = FileRead($aFileList[$i])
			_StripEmptyLines($sData)
			_StripWhitespace($sData)

			; Parse the TXT doc's description and syntax lines.
			$aArray = StringRegExp($sData, '^(?:' & $sType & '\R(\V*)\R*' & $HELP_SECTION_DESCRIPTION & '\R(\V*)\R*' & $HELP_SECTION_SYNTAX & '\R(\V*)\R(\V*))', $STR_REGEXPARRAYGLOBALMATCH)
			If Not @error And UBound($aArray) = $FUNCTION_PARAMS_MAX Then
				$aArray[$FUNCTION_PARAMS_TITLE] = StringRegExpReplace($aArray[$FUNCTION_PARAMS_TITLE], '(?:\.{3}|\h*/\h*)', $ASCII_ACK)
				$aKeywords = StringSplit($aArray[$FUNCTION_PARAMS_TITLE], $ASCII_ACK, $STR_STRINGSPLIT)
				For $j = 0 To UBound($aKeywords) - 1
					$hKeywords.Item($aKeywords[$j]) = $aKeywords[$j] ; Keyword list.
				Next
			EndIf
		Next

		$aKeywords = $hKeywords.Keys()
		_ArraySort($aKeywords)

		Local $sKeywordList = ''
		For $i = 0 To UBound($aKeywords) - 1
			$sKeywordList &= $aKeywords[$i] & @CRLF ; Keyword list.
		Next

		_StripEmptyLinesEndOfFile($sKeywordList) ; Strip EOLs.

		; Write the output.
		_FileWrite($sKeywordListOut, $sKeywordList)
	EndIf

	Return True
EndFunc   ;==>_GenerateKeywords

; ------------------------------------------------------------------------------
; Generate a list of macros.
; ------------------------------------------------------------------------------
Func _GenerateMacros($sMacroPath, $sMacroListOut, $sMacroFullOut)
	Local $aFileList = _FileListToArray($sMacroPath, '*.htm')
	If @error Then Return False

	Local $sData = ''
	; Read the macro files.
	For $i = 1 To UBound($aFileList) - 1
		$sData &= FileRead($sMacroPath & '\' & $aFileList[$i])
	Next

	Local $aMacroList = 0, _
			$hMacroList = 0
	; Generate a 2d array of macros.
	_MacroListToArray($sData, $aMacroList, $hMacroList)

	Local $sMacroList = '', $sMacroListFull = ''
	For $i = 0 To UBound($aMacroList) - 1
		$sMacroList &= $aMacroList[$i][$MACROLISTTOARRAY_NAME] & @CRLF
		$sMacroListFull &= $aMacroList[$i][$MACROLISTTOARRAY_NAME] & $ASCII_ACK & $aMacroList[$i][$MACROLISTTOARRAY_DESCRIPTION] & $ASCII_ACK & @CRLF
	Next
	_StripEmptyLinesEndOfFile($sMacroList)
	_StripEmptyLinesEndOfFile($sMacroList)
	_StripEmptyLinesEndOfFileACK($sMacroListFull)

	; Write the output.
	_FileWrite($sMacroListOut, $sMacroList)

	; Write the output.
	_FileWrite($sMacroFullOut, $sMacroListFull)

	Return True
EndFunc   ;==>_GenerateMacros

